const path = require("path");
const express = require("express");
const router = express.Router();
const adminFilePath = path.join(__dirname, "../src/views/admin.ejs");
const jwt = require("jsonwebtoken");
const pool = require("../lib/connect");

function parseCookies(request) {
  const list = {};
  const cookieHeader = request.headers?.cookie;
  if (!cookieHeader) return list;

  cookieHeader.split(`;`).forEach(function (cookie) {
    let [name, ...rest] = cookie.split(`=`);
    name = name?.trim();
    if (!name) return;
    const value = rest.join(`=`).trim();
    if (!value) return;
    list[name] = decodeURIComponent(value);
  });

  return list;
}

function authenticate(req, res, next) {
  const token = parseCookies(req).admin;
  if (token) {
    jwt.verify(token, "secret", (err, decoded) => {
      if (err) {
        res.redirect("/login");
      } else {
        req.user = decoded;
        next();
      }
    });
  } else {
    res.redirect("/login");
  }
}

router.get("/", authenticate, (req, res, next) => {
  res.render(adminFilePath);
});

router.get("/rutas", authenticate, (req, res) => {
  const sqlQuery = `SELECT rutas.id AS ruta_id,
  rutas.titulo, rutas.autor, rutas.descripcion, rutas.lugar,
  rutas.likes, rutas.ruta, rutas.distancia, rutas.clima,
  rutas.terreno, rutas.dificultad, rutas.altura, rutas.tipo_ruta,
  rutas.moto, rutas.tiempo, rutas.deptos, rutas.salida,
  rutas.llegada, rutas.lat, rutas.lng, video.url_video,
  JSON_ARRAYAGG(comentarios.comentario) AS comentarios,
  imagenes.url AS imagen_url
FROM rutas
LEFT JOIN video ON rutas.id = video.id_ruta
LEFT JOIN imagenes ON rutas.id = imagenes.id_ruta 
LEFT JOIN (
SELECT id_ruta, comentario
FROM comentarios
) AS comentarios ON rutas.id = comentarios.id_ruta
GROUP BY ruta_id , video.url_video ,imagenes.url`;
  pool.query(sqlQuery, (err, result) => {
    if (err) {
      console.log(err);
    } else {
      if (!result || !result.length) {
        res.status(404).json("No se encontraron rutas");
      } else {
        res.status(200).json(result);
      }
    }
  });
});

router.post("/rutas", authenticate, (req, res) => {
  const data = req.body;
  const { videos, imgs, portada_v, portada } = data;
  console.log(videos, imgs, portada, portada_v);
  try {
    let sql = `INSERT INTO rutas (titulo,autor,descripcion,lugar,ruta,distancia,clima,terreno,dificultad,altura,tipo_ruta,moto,tiempo,deptos,salida,llegada,lat,lng) 
    VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`;
    pool.query(
      sql,
      [
        data.titulo,
        data.autor,
        data.descripcion,
        data.lugar,
        data.ruta,
        data.distancia,
        data.clima,
        data.terreno,
        data.dificultad,
        data.altura,
        data.tipo_ruta,
        data.moto,
        data.tiempo,
        data.deptos,
        data.salida,
        data.llegada,
        data.lat,
        data.lng,
      ],
      (err, result) => {
        if (!err) {
          const lastInsertId = result.insertId;
          let sqlVideo = `INSERT INTO video (portada_video, url_video, id_ruta) 
                        VALUES (?,?,?)`;
          let sqlImg = `INSERT INTO imagenes (portada, url, id_ruta) 
                        VALUES (?,?,?)`;
          if (portada || imgs) {
            pool.query(sqlImg, [portada, JSON.stringify(imgs), lastInsertId], (imgErr, imgResult) => {
              if (imgErr) {
                console.error("Error al insertar imágenes: ", imgErr);
                // Manejo de error al insertar imágenes
                res.status(400).json("Error al guardar las imágenes");
              } else {
                // Se completó la inserción de imágenes
                // Puedes enviar una respuesta aquí si es necesario
              }
            });
          }
          // Verifica si se proporcionaron videos
          if (portada_v || videos) {
            pool.query(
              sqlVideo,
              [portada_v, JSON.stringify(videos), lastInsertId],
              (videoErr, videoResult) => {
                if (videoErr) {
                  console.error("Error al insertar videos: ", videoErr);
                  // Manejo de error al insertar videos
                  res.status(400).json("Error al guardar los videos");
                } else {
                  // Se completó la inserción de videos
                  // Puedes enviar una respuesta aquí si es necesario
                }
              }
            );
          }
          // Si no se proporcionaron ni imágenes ni videos
          if (!portada && !imgs && !portada_v && !videos) {
            res.status(200).json("Ruta guardada con éxito (sin imágenes ni videos)");
          } else {
            res.status(200).json("Ruta y/o archivos multimedia guardados con éxito");
          }
        }
      }
    );
  } catch (err) {
    console.log(err);
    res.status(500).json("error al crear la ruta");
  }
});

router.get("/rutas/:id", authenticate, (req, res) => {
  const id = parseInt(req.params.id);
  const sqlQuery = `SELECT rutas.id AS ruta_id,
                      rutas.titulo, rutas.autor, rutas.descripcion, rutas.lugar,
                      rutas.likes, rutas.ruta, rutas.distancia, rutas.clima,
                      rutas.terreno, rutas.dificultad, rutas.altura, rutas.tipo_ruta,
                      rutas.moto, rutas.tiempo, rutas.deptos, rutas.salida,
                      rutas.llegada, rutas.lat, rutas.lng, video.url_video,
                      JSON_ARRAYAGG(comentarios.comentario) AS comentarios,
                      imagenes.url AS imagen_url
                    FROM rutas
                    LEFT JOIN video ON rutas.id = video.id_ruta
                    LEFT JOIN imagenes ON rutas.id = imagenes.id_ruta 
                    LEFT JOIN (
                    SELECT id_ruta, comentario
                    FROM comentarios
                    ) AS comentarios ON rutas.id = comentarios.id_ruta
                    WHERE rutas.id = ?
                    GROUP BY ruta_id , video.url_video ,imagenes.url`;
  pool.query(sqlQuery, id, (err, result) => {
    if (err) {
      console.log(err);
    } else {
      if (!result || !result.length) {
        res.status(404).json("No se encontraron rutas");
      } else {
        res.status(200).json(result);
      }
    }
  });
});

router.delete("/rutas/:id", authenticate, (req, res, next) => {
  const id = parseInt(req.params.id);
  const sql = `DELETE FROM rutas WHERE id = ${id}`;
  pool.query(sql, (err, result) => {
    if (err) {
      res.status(400).json(err);
    } else {
      res.status(200).json("La ruta fue eliminada");
    }
  });
});

router.patch("/rutas/:id", authenticate, (req, res) => {
  const updateData = req.body;
  const id = parseInt(req.params.id);
  console.log(updateData, id);
  let sqlUpdate = "UPDATE rutas SET ? WHERE id = ?";
  pool.query(sqlUpdate, [updateData, id], (err, result) => {
    if (!err) {
      console.log(result);
      res.status(200).json("Ruta actualizada");
    } else {
      console.log(err);
      res.status(404).json("Error en la consulta");
    }
  });
});

router.post("/comments", authenticate, (req, res) => {
  const { user, comment, id_ruta } = req.body;

  let sqlComment = `INSERT INTO comentarios ( nombre_user, comentario, id_ruta )
                  VALUES (?,?,?)`;
  pool.query(sqlComment, [user, comment, id_ruta], (err, response) => {
    if (!err) {
      console.log(response);
      res.status(200).json("comentario añadido");
    } else {
      console.log(err);
    }
  });
});

router.delete("/comments/:id", authenticate, (req, res) => {
  const id = parseInt(req.params.id);
  const sqlComments = `DELETE FROM comentarios WHERE id_coment = ?`;
  pool.query(sqlComments, id, (err, result) => {
    if (!err) {
      console.log(result);
      res.status(200).json("Comentario eliminado con exito");
    } else {
      console.log(err);
      res.status(403).json("Error al eliminar el comentario");
    }
  });
});

router.patch("/images/:id", authenticate, (req, res) => {
  let id = parseInt(req.params.id);
});

router.patch("/videos/:id", authenticate, (req, res) => {
  let id = parseInt(req.params.id);
});

module.exports = router;

let insert = {
  titulo: "Ruta de las Cataratas",
  autor: "Juan Pérez",
  descripcion: "Explora una emocionante ruta llena de cataratas y cascadas impresionantes.",
  lugar: "Selva Amazónica",
  ruta: "Inicia en Iquitos y finaliza en Tarapoto, siguiendo el río y descubriendo maravillas naturales.",
  distancia: "900 km",
  clima: "Tropical, con lluvias frecuentes durante la temporada de lluvias.",
  terreno: "Caminos fluviales y senderos selváticos.",
  dificultad: "Media",
  altura: "Hasta 300 msnm",
  tipo_ruta: "Aventura",
  moto: "Honda CRF 450",
  tiempo: "8 días",
  deptos: "Loreto, San Martín",
  salida: "2023-10-15",
  llegada: "2023-10-22",
  lat: -3.7492,
  lng: -73.2538,
  portada: "ruta_cataratas.jpg",
  imgs: ["imagen1.jpg", "imagen2.jpg", "imagen3.jpg"],
  portada_v: "portada_video.jpg",
  videos: ["video1.mp4", "video2.mp4"],
};
