// Obtén todas las elementos con la clase "lazy-background"
const lazyBackgrounds = document.querySelectorAll(".lazy-background");
let windowWidth = window.innerWidth;
// Función para cargar la imagen de fondo
function loadBackgroundImage(element) {
  const imageUrl = element.getAttribute("data-src") || element.getAttribute("data-background");
  const imageUrl2 = element.getAttribute("data-src") || element.getAttribute("data-bgrsp");
  let path = window.location.origin;
  element.style.transition = '.5s ease-in-out'

  if (imageUrl) {
    let urlCompleta = path + imageUrl; // Combina la URL base con la URL de la imagen
    const img = new Image();
    img.src = urlCompleta;

    img.onload = function () {
      if (windowWidth <= 576) {
        // Si el ancho es menor que 768px, carga un fondo específico para pantallas pequeñas
        urlCompleta = path + imageUrl2;
      } else {
        // Si el ancho es mayor o igual a 768px, carga el fondo regular
        urlCompleta = path + imageUrl;
      }
      // Verifica si es una imagen de fondo o regular y aplica la URL completa en consecuencia
      if (element.classList.contains("lazy-background")) {
        element.style.backgroundColor = "inherit";
        element.style.backgroundImage = 'url("' + urlCompleta + '")';
      } else {
        element.setAttribute("src", urlCompleta);
      }
    };
  }
}

// Función para actualizar windowWidth cuando cambia el tamaño de la ventana
function updateWindowWidth() {
  windowWidth = window.innerWidth;
}

// Agregar un evento "resize" al objeto window
window.addEventListener("resize", updateWindowWidth);

// Función para cargar las imágenes de fondo de todos los elementos
function loadLazyBackgrounds() {
  lazyBackgrounds.forEach(function (element) {
    loadBackgroundImage(element);
  });
}

// Cargar las imágenes de fondo cuando se cargue la página o cuando sea necesario
window.addEventListener("DOMContentLoaded", () => {
  loadLazyBackgrounds();
});

window.addEventListener("resize", () => {
  loadLazyBackgrounds();
});
